from functools import wraps

from .exceptions import MsbAuthExceptions


def api_permissions(*permission_classes, _all: bool = False,**kwargs):
	def decorator_function(_decorated_function):
		@wraps(_decorated_function)
		def wrapper_function(cls, request, *args, **kwargs):
			override_validation = kwargs.get('override',True)
			api_permission_is_validated = getattr(cls,'_api_permission_is_validated',False)

			if override_validation and not api_permission_is_validated:
				evaluator = all if _all else any
				if not evaluator([permission().has_permission(request, cls) for permission in permission_classes]):
					raise MsbAuthExceptions.UnauthorizedAccess
				cls._api_permission_is_validated = True
			return _decorated_function(cls, *args, **dict(request=request, **kwargs))

		return wrapper_function

	return decorator_function


def require_role(*role_ids):
	def decorator_function(_decorated_function):
		@wraps(_decorated_function)
		def wrapper_function(cls, request, *args, **kwargs):
			if not any([request.user.has_role(role_id) for role_id in role_ids]):
				raise MsbAuthExceptions.UnauthorizedAccess

			_kwargs = dict(request=request, **kwargs)
			return _decorated_function(cls, *args, **_kwargs)

		return wrapper_function

	return decorator_function





from os import environ, path, listdir

from django.conf import settings
from django.core.servers.basehttp import WSGIServer
from django.test.testcases import (LiveServerThread, QuietWSGIRequestHandler)

from . import constants as const


class LiveServerThreadWithReuse(LiveServerThread):
	"""
	This miniclass overrides _create_server to allow port reuse. This avoids creating
	"address already in use" errors for tests that have been run subsequently.
	"""

	def _create_server(self):
		return WSGIServer((self.host, self.port), QuietWSGIRequestHandler, allow_reuse_address=True, )


class TestConfig:
	load_default_fixtures: bool = True
	load_test_fixtures: bool = True

	@property
	def default_fixture_dirs(self):
		_fixture_dirs: list = []
		if self.load_default_fixtures:
			_fixture_dirs.append(const.DEFAULT_FIXTURES_DIR)

			if self.load_test_fixtures:
				_fixture_dirs.append(const.TEST_FIXTURES_DIR)

			if len(_fixture_dirs) > 0:
				_declared_fixture_dirs = settings.FIXTURE_DIRS if isinstance(settings.FIXTURE_DIRS, list) else []
				_fixture_config = {path.basename(i): i for i in _declared_fixture_dirs}
				_fixture_dirs = [_fixture_config.get(i) for i in _fixture_dirs if i in _fixture_config.keys()]

		return _fixture_dirs

	def _load_fixtures_in_db(self, *fixtures):
		from django.core import management
		management.call_command('loaddata', *fixtures)

	def __load_default_fixtures(self):
		for fixture_dir in self.default_fixture_dirs:
			_fixture_files = [path.basename(file) for file in sorted(listdir(fixture_dir))]
			self._load_fixtures_in_db(*_fixture_files)

	def __init__(self):
		self.__load_default_fixtures()

	def get(self, key: str = None, default=None):
		return environ.get(f"TEST_{str(key).upper()}", default=default)

	@property
	def auth_credentials(self):
		return dict(
			user=self.get('AUTH_USER'),
			password=self.get('AUTH_PASSWORD'),
			auth_type=self.get('AUTH_TYPE')
		)



from django.test import LiveServerTestCase

from .core import (LiveServerThreadWithReuse, TestConfig, const)


class UnitTestConfig(TestConfig):
	pass


class UnitTest(LiveServerTestCase, UnitTestConfig):
	databases = const.DEFAULT_TEST_DATABASES
	port = const.DEFAULT_TEST_SERVER_PORT
	server_thread_class = LiveServerThreadWithReuse

	def __init__(self, *args, **kwargs):
		LiveServerTestCase.__init__(self, *args, **kwargs)
		UnitTestConfig.__init__(self)



import logging

from ._exception import AppException


class MsbExceptionHandler:

	@staticmethod
	def handle_exception(e):
		return MsbExceptionHandler.handle_error(e)
		logging.exception(e)

	@staticmethod
	def handle_error(e):
		logging.error(f"Error : {e}")

	@staticmethod
	def raise_exceptions(e: AppException|Exception, alternate_exc: AppException|Exception = None, silent=False):
		if not silent:
			raise (e if isinstance(e, AppException) else alternate_exc)



from abc import ABC, abstractmethod

from .dataclasses import GeneratedFile


class FileInterface(ABC):
	"""Save Function will generate the file to the path given during function call"""

	@abstractmethod
	def save(self, filename: str, location: str) -> GeneratedFile:
		pass

	"""set_content() Function will store the content values that will used to save within the file"""

	@abstractmethod
	def set_content(self, content: str):
		pass

	"""set_file_overwrite() Function will be used if someone want to overwrite the existed file"""

	@abstractmethod
	def set_file_overwrite(self):
		pass

	@abstractmethod
	def set_protection(self):
		pass


class FileBase:
	content: str = None
	file_overwrite: bool = False

	def set_content(self, content: str):
		self.content = content
		return self

	def set_file_overwrite(self):
		self.file_overwrite = True
		return self

	def set_protection(self):
		return self



from msb.exceptions import AppException


class MsbConfigException():
	class BaseDirDoesNotExists(AppException):
		_message = "Root Directory Does Not Exists."

	class ConfigFileDoesNotExist(AppException):
		_message = "Configuration File Does Not Exists."

	class ConfigurationLoadingFailed(AppException):
		_message = "Failed To Load Configuration From envutil File."



from django.utils.translation import gettext_lazy as _
from rest_framework.fields import CharField
from rest_framework_simplejwt.serializers import (TokenObtainPairSerializer, TokenVerifySerializer)

from msb.cipher import Cipher
from .constants import (
	TokenFieldNames as _tfn, AUTH_REQUEST_TYPE_FIELD_NAME, AUTH_REQUEST_USER_FIELD_NAME,
	AUTH_TOKEN_ACCESS_TYPE_ISR, AUTH_REQUEST_TYPE_ISR
)
from .session import SessionData
from .users import TokenUser


class MsbJwtTokenObtainSerializer(TokenObtainPairSerializer):
	"""Customizes JWT default Serializer to add more information about user"""
	username_field = AUTH_REQUEST_USER_FIELD_NAME
	default_error_messages = {
		'no_active_account': _('User Authentication Failed.')
	}

	def __init__(self, *args, **kwargs):
		super(MsbJwtTokenObtainSerializer, self).__init__(*args, **kwargs)
		self.fields[AUTH_REQUEST_TYPE_FIELD_NAME] = CharField(required=True)
		self.fields[AUTH_REQUEST_USER_FIELD_NAME] = CharField(required=True)

	@classmethod
	def format_token_data(cls, token_data, user: TokenUser):
		return token_data

	@classmethod
	def get_token(cls, user: TokenUser = None):
		try:
			token_data = super().get_token(user)
			token_data[_tfn.ID] = user.token.get(_tfn.USER_ID)
			token_data[_tfn.IP] = user.token.get(_tfn.IP)
			token_data[_tfn.IS_VALID] = user.token.get(_tfn.IS_VALID)
			token_data[_tfn.SESSION] = user.token.get(_tfn.SESSION).to_json()
			token_data[_tfn.OWNER] = Cipher.encrypt(f"{user.token.get(_tfn.OWNER)}@{user.token.get(_tfn.IP)}")
			token_data[_tfn.ACCESS_TYPE] = Cipher.encrypt(user.token.get(_tfn.ACCESS_TYPE))

			return cls.format_token_data(token_data, user)
		except Exception as e:
			return None


class MsbJwtTokenVerifySerializer(TokenVerifySerializer):
	pass


class MsbIntraServiceTokenObtainSerializer(MsbJwtTokenObtainSerializer):
	"""Customizes JWT default Serializer to add more information about user"""

	@classmethod
	def format_token_data(cls, token_data, user: TokenUser = None):
		super(MsbIntraServiceTokenObtainSerializer, cls).format_token_data(token_data=token_data, user=user)
		token_data['token_type'] = 'access'
		return token_data

	@classmethod
	def create_intra_service_request_token(cls, name: str, email: str, id: str, path: str, env: str, **kwargs):
		session_data = SessionData(id=id, email=email, username=name, work_id=path)
		session_data.set_auth_type(AUTH_REQUEST_TYPE_ISR)
		session_data.set_is_authenticated(True)

		for sub in kwargs.get('subscriptions', []):
			session_data.add_subscription(sub)

		token_data = dict(
			userid=session_data.id,
			is_valid=session_data.is_authenticated,
			owner=name, ip=None, env=env,
			access_type=AUTH_TOKEN_ACCESS_TYPE_ISR,
		)
		return cls.get_token(TokenUser(tokendata=dict(**token_data, session=session_data)))



from msb.cipher import Cipher
from msb.dataclasses import Singleton
from msb.exceptions import MsbExceptionHandler


class MsbService(MsbExceptionHandler, metaclass=Singleton):
	service_name: str = None

	@property
	def cipher(self) -> Cipher:
		return Cipher

	@property
	def service_name_value(self) -> str | None:
		try:
			if not self.service_name:
				import re
				class_name = re.split('(?<=.)(?=[A-Z])', self.__class__.__name__.replace("Service", ""))
				return " ".join(class_name)
			return self.service_name
		except Exception:
			return None

	def __str__(self):
		return f"<MsbService: {self.service_name_value}>"

	def __repr__(self):
		return self.__str__()



import logging
from argparse import ArgumentParser

from msb.env import Config, NameConst, PathConst

from .constants import (REGEX_TO_SELECT_ENV_VARIABLE, REGEX_TO_REPLACE_ENV_VARIABLE)
from .django import (DjangoFixtures)
from .funcs import (
	require_django, run_management_command, drop_database_tables
)


def build_prod_env_file_from(dev_env_file_path: str = None, prod_env_file_path: str = None):
	import os, re

	dev_env_file_path = dev_env_file_path or PathConst.DEV_ENV_FILE_PATH
	prod_env_file_path = prod_env_file_path or PathConst.PROD_ENV_FILE_PATH

	if os.path.isfile(dev_env_file_path) and os.path.isfile(prod_env_file_path):
		with open(dev_env_file_path, "r+") as dev_env_file:
			dev_env_file_content = dev_env_file.read()

		prod_env_file_content = re.sub(
			REGEX_TO_SELECT_ENV_VARIABLE, REGEX_TO_REPLACE_ENV_VARIABLE, dev_env_file_content
		)

		with open(prod_env_file_path, "w+") as prod_env_file:
			prod_env_file.write(prod_env_file_content)


def _drop_database_task(**kwargs):
	"""[ -d=<str:'db_name'> ] : Drops all tables from the databse"""
	if kwargs.get("env", None) not in NameConst.DEV_ENV_NAMES_LIST:
		return logging.warning(msg=f"Dropping Tables only allowed in {NameConst.DEV_ENV_NAMES_LIST} Enviroment")
	if not (db := kwargs.get("-d", None)):
		return logging.warning(msg="Please specify a database name")

	drop_database_tables(db_name=db)


def _load_database_fixtures_task(env=None, **kwargs):
	""": Loads Database Fixtures for the current env."""
	_fixture_dirs = [
		NameConst.DEFAULT_FIXTURE_DIR_NAME,
		NameConst.TEST_FIXTURE_DIR_NAME if Config.is_dev_or_test_env() else NameConst.PROD_FIXTURE_DIR_NAME
	]
	database_fixtures = DjangoFixtures().load(*_fixture_dirs)


def _fresh_db_setup_task(**kwargs):
	"""[ -b=<:bool=False> build Migration ] [ -l=<:bool=True>, Load Fixtures ] [ -d=<str:'default'> ]
	: Drops all tables,	Migrate created	migrations & loads db fixtures."""
	if kwargs.get("env", None) not in NameConst.DEV_ENV_NAMES_LIST:
		raise ValueError(f"This Task can Only Be Run For {NameConst.DEV_ENV_NAMES_LIST} Environment")
	_drop_database_task(**{"-d": kwargs.get('-d', NameConst.DEFAULT_DATABASE_NAME), **kwargs})
	_migrate_task(**dict(**kwargs))
	if kwargs.get("-l", True):
		_load_database_fixtures_task(**kwargs)


def _make_migrations_task(**kwargs):
	""": Creates db migrations for all apps."""
	for app in kwargs.get('apps', []):
		run_management_command("makemigrations", app)


def _migrate_task(**kwargs):
	"""[ -b=<bool> build Migration ] : Migrate all apps."""
	if kwargs.get("-b", False):
		_make_migrations_task(**kwargs)
	run_management_command("migrate", "--no-input")


@require_django
class MsbSetupTask:
	__task_details: dict = {
		"load_fixtures": _load_database_fixtures_task,
		"makemigrations": _make_migrations_task,
		"migrate": _migrate_task,
		"clean_db": _drop_database_task,
		"reset_db": _fresh_db_setup_task
	}
	_kwargs: dict = {}

	@property
	def task_names(self):
		return list(self.__task_details.keys())

	@property
	def usage_text(self):
		_task_list = [f"[ -t {k} ] {v.__doc__}\n" for k, v in self.__task_details.items()]
		return f"python setup.py -t task_name arg1=value1  arg2=value2 if any.\n{''.join(_task_list)}\n\n\n"

	@property
	def env(self):
		return self._kwargs.get('env', None)

	def __get_user_inputs(self):
		parser = ArgumentParser(prog='Msb Application Setup Task', usage=self.usage_text)
		parser.add_argument('-t', choices=self.task_names, help='Task Name', required=True)
		return parser.parse_known_args()

	def __init__(self, for_apps: list, for_dbs: list, **kwargs):
		self._kwargs = {"env": Config.env_name(), "apps": for_apps, "dbs": for_dbs, **kwargs}

	def run(self):
		try:
			input_args, task_args = self.__get_user_inputs()
			input_task_name = input_args.t

			if not callable(task_name := self.__task_details.get(input_task_name, None)):
				raise ValueError(f"Invalid Task Name {task_name}")

			self._kwargs.update({i.split("=")[0]: i.split("=")[1] for i in task_args if "=" in i})

			return task_name(**self._kwargs)
		except KeyboardInterrupt:
			logging.warning("Keyboard interrupt occurred.")
		except Exception as e:
			logging.exception(e)



import logging
from datetime import datetime, timedelta

import pytz

from .constants import *


def current_time(frmt=DEFAULT_TIME_FORMAT):
	return datetime.now().time().strftime(frmt)


def current_date(frmt=DEFAULT_DATE_FORMAT):
	return datetime.now().date().strftime(frmt)


def current_timestamp(frmt=TIMESTAMP_FORMAT):
	return datetime.now().strftime(frmt)


def timestamp_diff(latest=None, old=None, return_in='sec|min|hr|day'):
	diff_data = dict(sec=0, min=0, hr=0, day=0)
	return_in = 'sec' if return_in not in ['sec', 'min', 'hr', 'day'] else return_in
	try:
		latest = datetime.strptime(latest, TIMESTAMP_FORMAT)
		old = datetime.strptime(old, TIMESTAMP_FORMAT)
		diff = (latest - old)
		diff_data['sec'] = diff.total_seconds()
		diff_data['min'] = (diff_data.get('sec') / 60)
		diff_data['hr'] = (diff_data.get('min') / 60)
		diff_data['day'] = diff.days
	# print (diff_data)
	except Exception as e:
		logging.exception(e)
	return diff_data.get(return_in)


def create_datetime(inp_dtime=None, hours=0, mins=0, secs=0, days=0, inp_fmt=TIMESTAMP_FORMAT, out_fmt=None):
	try:
		out_fmt = inp_fmt if out_fmt is None else out_fmt
		inp_dtime = str(datetime.now().strftime(inp_fmt)) if inp_dtime is None else inp_dtime
		return (
			datetime.strptime(inp_dtime, inp_fmt) +
			timedelta(
				hours=hours, days=days,
				minutes=mins,
				seconds=secs
			)
		).strftime(out_fmt)
	except Exception as e:
		print(e)


def parse_datetime(inp_dtime=None, inp_fmt=TIMESTAMP_FORMAT):
	result = None
	try:
		result = datetime.strptime(str(inp_dtime), inp_fmt)

	except Exception as e:
		logging.exception(e)
	return result


def convert_to_utc(inp_dtime=None, inp_fmt=TIMESTAMP_FORMAT, out_fmt=None, ist_time_diff='0:00'):
	try:
		out_fmt = inp_fmt if out_fmt is None else out_fmt
		dt = parse_datetime(str(inp_dtime), inp_fmt=inp_fmt)
		hour_diff, mins_diff = ist_time_diff.split(":")
		utc_time = create_datetime(
			dt.__str__(), hours=-int(hour_diff), mins=-int(mins_diff),
			inp_fmt=inp_fmt
		)
		return parse_datetime(utc_time, inp_fmt=inp_fmt).strftime(out_fmt)


	except Exception as e:
		logging.exception(e)


def timezone_offset(timezone=DEFAULT_TIMEZONE):
	offset = 0
	try:
		offset = (datetime.now(pytz.timezone(timezone)).utcoffset().total_seconds() / 3600)
	except Exception as e:
		logging.exception(e)
	return offset



from django.db import models

from .model_fields import EncryptedString
from .msb_model import MsbModel
from .msb_model import MsbModelManager


class ConfigurationModelManager(MsbModelManager):

	def get_config(self, name: str, map_to=None):
		_config_list = []
		try:
			_config_list = self.get_queryset().filter(name=name).all()
		except Exception:
			pass
		return map_to(*_config_list) if callable(map_to) else _config_list


# Configuration model
class Configuration(MsbModel):
	_list_field_names = ["name", "label", "value"]
	_identifier_field = "name"

	# meta data
	class Meta:
		abstract = True
		indexes = [models.Index(fields=['name', 'key'])]
		unique_together = ("name", "key")

	# Model fields
	name = models.CharField(max_length=100, db_column='name')
	key = models.CharField(max_length=255, db_column='key')
	value = EncryptedString(db_column='value')
	label = models.CharField(max_length=255, db_column='label', null=True)
	field_type = models.CharField(max_length=255, db_column='field_type', null=True)

	# assign a custom manager to the model
	objects = ConfigurationModelManager()

	def __str__(self):
		return f"<{self.__class__.__name__} [{self.name}]: {self.key}>"



from __future__ import annotations


class RequestInfo:
	__request_info = None

	def __init__(self, meta=None):
		self.__request_info = meta

	def get(self, key: str = None, default=None):
		val = self.__request_info.get(key)
		return val if val is not None else default

	@property
	def cookie(self):
		return self.__request_info.get('HTTP_COOKIE')

	@property
	def path(self) -> str:
		return self.__request_info.get('PATH_INFO')

	@property
	def ip(self):
		return self.__request_info.get('REMOTE_ADDR')

	@property
	def method(self) -> str:
		return self.__request_info.get('REQUEST_METHOD')

	@property
	def script(self) -> str:
		return self.__request_info.get('SCRIPT_NAME')

	@property
	def server(self) -> str:
		return self.__request_info.get('SERVER_NAME')

	@property
	def port(self) -> int:
		return int(self.__request_info.get('SERVER_PORT'))

	@property
	def protocol(self) -> str:
		return self.__request_info.get('SERVER_PROTOCOL')

	@property
	def content_type(self) -> str:
		return self.__request_info.get('CONTENT_TYPE')

	@property
	def query_string(self) -> str:
		return self.__request_info.get('QUERY_STRING')

	@property
	def authorization(self) -> str:
		return self.__request_info.get('HTTP_AUTHORIZATION')


class RequestHeaders:
	__headers = None

	def __init__(self, headers=None):
		self.__headers = headers

	@property
	def user_agent(self) -> str:
		return self.__headers.get('User-Agent')



from typing import List, Dict

from django.core.exceptions import FieldDoesNotExist
from django.db import (IntegrityError, )
from django.db.models.query import QuerySet

from msb.dataclasses import SearchParameter
from msb.db import constants as _const
from msb.db.models import MsbModel
from .exceptions import ApiServiceExceptions
from .msb_service import MsbService


class ApiService(MsbService):
	db_model: MsbModel = MsbModel
	exception_class_name: ApiServiceExceptions | None = None

	@property
	def exception_class(self) -> ApiServiceExceptions:
		if not isinstance(self.exception_class_name, ApiServiceExceptions):
			return ApiServiceExceptions
		return self.exception_class_name

	def get_meta_field_data(self, field_name: str, field_value) -> Dict:
		if hasattr(self.db_model, field_name) and str(field_value).isnumeric():
			return {field_name: field_value}
		return {}

	def __init__(self):
		if self.db_model is None or self.db_model._meta.model_name == MsbModel._meta.model_name:
			raise ApiServiceExceptions.InvalidDatabaseModel(resource=self.service_name_value)

	def search(self, params: SearchParameter) -> Dict:
		result = dict(count=0, records=[])
		try:
			model_query = params.get_query(self.db_model)
			result['count'] = model_query.count()
			model_query = model_query.all()[params.offset:params.offset + params.limit]
			result['records'] = [i.dict() for i in model_query]
		except FieldDoesNotExist as fde:
			raise ApiServiceExceptions.InvalidSearchField(resource=self.service_name_value)
		except Exception as e:
			self.raise_exceptions(e, ApiServiceExceptions.SearchOperationFailed(resource=self.service_name_value, errors=e))
		return result

	def create(self, *model_data_list, user_id=None) -> bool | MsbModel | List[MsbModel]:
		try:
			if len(model_data_list) == 0:
				raise ApiServiceExceptions.InvalidDataForCreateOperation(resource=self.service_name_value)

			created_by = self.get_meta_field_data(_const.COLUMN_NAME_CREATED_BY, user_id)

			if len(model_data_list) == 1:
				model_data = model_data_list[0]
				_model = self.db_model(**{**model_data, **created_by})
				_model.save()
				return _model
			else:
				_model_list = [self.db_model({**model_data, **created_by}) for model_data in model_data_list]
				self.db_model.objects.bulk_create(_model_list)
				return _model_list
		except IntegrityError as ie:
			raise ApiServiceExceptions.DuplicateEntry(resource=self.service_name_value)
		except Exception as e:
			self.raise_exceptions(e, ApiServiceExceptions.CreateOperationFailed(resource=self.service_name_value))

	def retrieve(self, pk=None, silent=False):
		try:
			if (pk := self.cipher.decipher(pk)) is None:
				raise ApiServiceExceptions.InvalidPk(resource=self.service_name_value)
			return self.db_model.objects.retrieve(pk=pk)
		except self.db_model.DoesNotExist:
			raise ApiServiceExceptions.ResourseDoesNotExists(resource=self.service_name_value)
		except Exception as e:
			self.raise_exceptions(e, ApiServiceExceptions.RetrieveOperationFailed(resource=self.service_name_value), silent)
		return None

	def list(self, limit: int = _const.DEFAULT_QUERY_LIMIT, offset: int = _const.DEFAULT_QUERY_OFFSET) -> QuerySet | None:
		try:
			offset = int(offset) if str(offset).isnumeric() else _const.DEFAULT_QUERY_OFFSET
			limit = int(limit) if str(limit).isnumeric() else _const.DEFAULT_QUERY_LIMIT

			fields = [
				i for i in self.db_model._list_field_names
				if (x := getattr(self.db_model, i, None)) and not isinstance(x, property)
			]
			data_set = self.db_model.objects.only(*fields).all()
			return data_set[offset:(limit + offset)] if len(fields) > 0 else None
		except Exception as e:
			self.raise_exceptions(e, ApiServiceExceptions.ListOperationFailed(resource=self.service_name_value))

	def update(self, pk=None, user_id=None, **model_data) -> bool:
		try:
			if (pk := self.cipher.decipher(pk)) is None:
				raise ApiServiceExceptions.InvalidPk(resource=self.service_name_value)

			if not (model_object := self.db_model.objects.filter(pk=pk)).exists():
				raise ApiServiceExceptions.ResourseDoesNotExists(resource=self.service_name_value)

			model_data.update(self.get_meta_field_data(_const.COLUMN_NAME_UPDATED_BY, user_id))

			if not (status := model_object.update(**model_data)):
				raise ApiServiceExceptions.UpdateOperationFailed(resource=self.service_name_value)
			return bool(status)
		except IntegrityError as ie:
			raise ApiServiceExceptions.DuplicateEntry(resource=self.service_name_value)
		except Exception as e:
			self.raise_exceptions(e, ApiServiceExceptions.UpdateOperationFailed(resource=self.service_name_value))

	def delete(self, pk=None, user_id=None) -> bool:
		try:
			if (pk := self.cipher.decipher(pk)) is None:
				raise ApiServiceExceptions.InvalidPk(resource=self.service_name_value)
			model_object = self.retrieve(pk=pk)
			delete_kwargs = self.get_meta_field_data(_const.COLUMN_NAME_DELETED_BY, user_id)
			if not (status := model_object.delete(**delete_kwargs)):
				raise ApiServiceExceptions.DeleteOperationFailed(resource=self.service_name_value)
			return bool(status)
		except Exception as e:
			self.raise_exceptions(e, ApiServiceExceptions.DeleteOperationFailed(resource=self.service_name_value))



CRUD_URL_PK = '/<str:pk>'
CRUD_URL_PK_NAME = "pk"
MAX_FETCH_LIMIT = 100
DEFAULT_LOGGER_NAME = 'root'


class REQUEST_METHODS:
	POST = "post"
	PUT = "put"
	GET = "get"
	DELETE = "delete"

	ALL = [POST, PUT, GET, DELETE]


class CrudActions:
	create = 'create'
	retrieve = 'retrieve'
	list = 'list'
	search = 'search'

	update = 'update'
	bulk_update = 'bulk_update'

	delete = 'delete'
	bulk_delete = 'bulk_delete'

	all = [create, retrieve, update, delete, bulk_delete, bulk_update, list]
	single_read = [retrieve, ]
	create_only = [create]
	create_and_update = [create, update, bulk_update]
	full_read = [retrieve, list]
	full_write = [*create_and_update, delete, bulk_delete]


class CrudMethods:
	create = 'create'
	retrieve = 'retrieve'
	list = 'list'
	search = 'search'
	update = 'update'
	bulk_update = 'bulk_update'
	delete = 'delete'
	bulk_delete = 'bulk_delete'
	not_found = 'api_not_found'
